@@ -55,6 +55,8 @@
 
 #include "gedit-marshal.h"
 
+#include <gtksourceview/gtksourceiter.h>
+
 #ifdef MAXPATHLEN
 #define GEDIT_MAX_PATH_LEN  MAXPATHLEN
 #elif defined (PATH_MAX)
@@ -86,6 +88,7 @@ enum {
 	SAVED,
 	LOADED,
 	READONLY_CHANGED,
+	CAN_FIND_AGAIN,
 	LAST_SIGNAL
 };
 
@@ -98,8 +101,10 @@ static void gedit_document_real_loaded			(GeditDocument *document);
 static void gedit_document_real_saved			(GeditDocument *document);
 static void gedit_document_real_readonly_changed	(GeditDocument *document,
 							 gboolean readonly);
+static void gedit_document_real_can_find_again		(GeditDocument *document);
 
-static gboolean	gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, 
+static gboolean	gedit_document_save_as_real (GeditDocument *doc, const gchar *uri,
+	       				     const GeditEncoding *encoding,	
 					     gboolean create_backup_copy, GError **error);
 static void gedit_document_set_uri (GeditDocument* doc, const gchar* uri);
 
@@ -114,6 +119,8 @@ static GHashTable* allocated_untitled_numbers = NULL;
 static gint gedit_document_get_untitled_number (void);
 static void gedit_document_release_untitled_number (gint n);
 
+static void gedit_document_set_encoding (GeditDocument *doc, const GeditEncoding *encoding);
+
 static gint
 gedit_document_get_untitled_number (void)
 {
@@ -199,6 +206,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 	klass->loaded 	    	= gedit_document_real_loaded;
 	klass->saved        	= gedit_document_real_saved;
 	klass->readonly_changed = gedit_document_real_readonly_changed;
+	klass->can_find_again	= gedit_document_real_can_find_again;
 
   	document_signals[NAME_CHANGED] =
    		g_signal_new ("name_changed",
@@ -240,11 +248,23 @@ gedit_document_class_init (GeditDocumentClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_BOOLEAN);
+
+	document_signals[CAN_FIND_AGAIN] =
+   		g_signal_new ("can_find_again",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GeditDocumentClass, can_find_again),
+			      NULL, NULL,
+			      gedit_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 }
 
 static void
 gedit_document_init (GeditDocument *document)
 {
+	const GeditEncoding *enc;
+	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	document->priv = g_new0 (GeditDocumentPrivate, 1);
@@ -256,16 +276,10 @@ gedit_document_init (GeditDocument *document)
 
 	document->priv->last_save_was_manually = TRUE;
 
-	if (gedit_prefs_manager_get_save_encoding () == 
-			GEDIT_SAVE_ORIGINAL_FILE_ENCODING_IF_POSSIBLE_NCL)
-	{
-		const gchar *encoding = NULL;
-		g_get_charset(&encoding);
-				
-		document->priv->encoding = g_strdup (encoding);
-	}
-	else
-		document->priv->encoding = NULL;
+	enc = gedit_encoding_get_utf8 ();
+		
+	document->priv->encoding = g_strdup (
+				gedit_encoding_get_charset (enc));			
 }
 
 static void
@@ -374,20 +388,22 @@ gedit_document_new (void)
  * Return value: a new document
  **/
 GeditDocument*
-gedit_document_new_with_uri (const gchar *uri, GError **error)
+gedit_document_new_with_uri (const gchar          *uri,  
+			     const GeditEncoding  *encoding, 
+			     GError              **error)
 {
  	GeditDocument *document;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (uri != NULL, NULL);
-
+	
 	document = GEDIT_DOCUMENT (g_object_new (GEDIT_TYPE_DOCUMENT, NULL));
 
 	g_return_val_if_fail (document->priv != NULL, NULL);
 	document->priv->uri = g_strdup (uri);
 
-	if (!gedit_document_load (document, uri, error))
+	if (!gedit_document_load (document, uri, encoding, error))
 	{
 		gedit_debug (DEBUG_DOCUMENT, "ERROR");
 
@@ -509,8 +525,9 @@ gedit_document_real_loaded (GeditDocument *document)
 					   "last_searched_text");
 	if (data != NULL)
 	{
-		gedit_document_set_last_searched_text (document,
-						       data);
+		if (document->priv->last_searched_text == NULL)
+			gedit_document_set_last_searched_text (document,
+							       data);
 
 		g_free (data);
 	}
@@ -519,8 +536,9 @@ gedit_document_real_loaded (GeditDocument *document)
 					   "last_replaced_text");
 	if (data != NULL)
 	{
-		gedit_document_set_last_replace_text (document,
-						      data);
+		if (document->priv->last_replace_text == NULL)
+			gedit_document_set_last_replace_text (document,
+							      data);
 
 		g_free (data);
 	}
@@ -542,6 +560,14 @@ gedit_document_real_readonly_changed (GeditDocument *document, gboolean readonly
 	g_return_if_fail (document != NULL);
 }
 
+static void 
+gedit_document_real_can_find_again (GeditDocument *document)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (document != NULL);
+}
+
 
 gchar*
 gedit_document_get_raw_uri (const GeditDocument* doc)
@@ -664,11 +690,16 @@ gedit_document_auto_save_timeout (GeditDocument *doc)
 static gboolean
 gedit_document_auto_save (GeditDocument* doc, GError **error)
 {
+	const GeditEncoding *encoding;
+	
 	gedit_debug (DEBUG_DOCUMENT, "");
 	
 	g_return_val_if_fail (doc != NULL, FALSE);
 
-	if (gedit_document_save_as_real (doc, doc->priv->uri, doc->priv->last_save_was_manually, NULL))
+	encoding = gedit_document_get_encoding (doc);
+		
+	if (gedit_document_save_as_real (doc, doc->priv->uri, encoding,
+					 doc->priv->last_save_was_manually, NULL))
 		doc->priv->last_save_was_manually = FALSE;
 
 	return TRUE;
@@ -685,7 +716,10 @@ gedit_document_auto_save (GeditDocument* doc, GError **error)
  * Return value: TRUE if the file is correctly loaded
  **/
 gboolean
-gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
+gedit_document_load (GeditDocument        *doc, 
+		     const gchar          *uri, 
+		     const GeditEncoding  *encoding,
+		     GError              **error)
 {
 	char* file_contents;
 	GnomeVFSResult res;
@@ -703,42 +737,87 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 	
 	if (res != GNOME_VFS_OK)
 	{
-		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, res,
-			gnome_vfs_result_to_string (res));
+		g_set_error (error, 
+			     GEDIT_DOCUMENT_IO_ERROR, 
+			     res,
+			     gnome_vfs_result_to_string (res));
 		return FALSE;
 	}
 
 	if (file_size > 0)
 	{
+		GError *conv_error = NULL;
 		gchar *converted_text;
 		gint len = -1;
 
-		if (g_utf8_validate (file_contents, file_size, NULL))
+		if (encoding == gedit_encoding_get_utf8 ())
 		{
-			converted_text = file_contents;
-			len = file_size;
-			file_contents = NULL;
+			if (g_utf8_validate (file_contents, file_size, NULL))
+			{
+				converted_text = file_contents;
+				len = file_size;
+				file_contents = NULL;
+			}
+			else
+			{
+				converted_text = NULL;
+				g_set_error (&conv_error, GEDIT_CONVERT_ERROR, 
+					GEDIT_CONVERT_ERROR_ILLEGAL_SEQUENCE,
+					"The file you are trying to open contain an invalid byte sequence.");
+			}
 		}
 		else
-			converted_text = gedit_convert_to_utf8 (file_contents,
-							file_size,
-							&doc->priv->encoding);
+		{
+			converted_text = NULL;
+			
+			if (encoding == NULL)
+			{
+				const GeditEncoding *enc;
+				gchar *charset;
 
+				charset = gedit_metadata_manager_get (uri, "encoding");
+
+				if (charset != NULL)
+				{
+					enc = gedit_encoding_get_from_charset (charset);
+
+					if (enc != NULL)
+					{	
+						converted_text = gedit_convert_to_utf8 (file_contents,
+										file_size,
+										&enc,
+										NULL);
+
+						if (converted_text != NULL)
+							encoding = enc;
+					}
+
+					g_free (charset);
+				}
+			}
+
+			if (converted_text == NULL)				
+				converted_text = gedit_convert_to_utf8 (file_contents,
+								file_size,
+								&encoding,
+								&conv_error);
+		}
+		
 		if (converted_text == NULL)
 		{
 			/* bail out */
-			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR,
-				     GEDIT_ERROR_INVALID_UTF8_DATA,
-				     _("Invalid UTF-8 data"));
+			if (conv_error == NULL)
+				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR,
+					     GEDIT_ERROR_INVALID_UTF8_DATA,
+					     _("Invalid UTF-8 data"));
+			else
+				g_propagate_error (error, conv_error);
 
 			g_free (file_contents);
 
 			return FALSE;
 		}
 
-		gedit_debug (DEBUG_DOCUMENT, "Document encoding: %s", 
-			     doc->priv->encoding == NULL ? "UTF-8 (Null)" : doc->priv->encoding);
-
 		gedit_document_begin_not_undoable_action (doc);
 		/* Insert text in the buffer */
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, 0);
@@ -756,6 +835,8 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 
 		gedit_document_end_not_undoable_action (doc);
 	}
+	else
+		encoding = gedit_encoding_get_utf8 ();
 
 	if (file_contents != NULL)
 		g_free (file_contents);
@@ -776,7 +857,9 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), FALSE);
 
 	gedit_document_set_uri (doc, uri);
-		
+
+	gedit_document_set_encoding (doc, encoding);
+
 	g_signal_emit (G_OBJECT (doc), document_signals[LOADED], 0);
 
 	return TRUE;
@@ -790,30 +873,49 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
-	g_return_val_if_fail (doc != NULL, FALSE);
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 	
 	stdin_data = gedit_utils_get_stdin ();
 
 	if (stdin_data != NULL && strlen (stdin_data) > 0)
 	{
+		GError *conv_error = NULL;
+		const GeditEncoding *encoding;
 		gchar *converted_text;
 
-		if (g_utf8_validate (stdin_data, -1, NULL))
+		if (gedit_encoding_get_current () != gedit_encoding_get_utf8 ())
 		{
-			converted_text = stdin_data;
-			stdin_data = NULL;
+			if (g_utf8_validate (stdin_data, -1, NULL))
+			{
+				converted_text = stdin_data;
+				stdin_data = NULL;
+
+				encoding = gedit_encoding_get_utf8 ();
+			}
+			else
+			{
+				converted_text = NULL;
+			}
 		}
 		else
+		{	
+			encoding = gedit_encoding_get_current ();
+				
 			converted_text = gedit_convert_to_utf8 (stdin_data,
-							-1,
-							&doc->priv->encoding);
-
+								-1,
+								&encoding,
+								&conv_error);
+		}	
+		
 		if (converted_text == NULL)
 		{
 			/* bail out */
-			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR,
-				     GEDIT_ERROR_INVALID_UTF8_DATA,
-				     _("Invalid UTF-8 data"));
+			if (conv_error == NULL)
+				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR,
+					     GEDIT_ERROR_INVALID_UTF8_DATA,
+					     _("Invalid UTF-8 data"));
+			else
+				g_propagate_error (error, conv_error);
 
 			g_free (stdin_data);
 
@@ -855,6 +957,8 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 gboolean 
 gedit_document_is_untouched (const GeditDocument *doc)
 {
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), TRUE);
+
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	return (doc->priv->uri == NULL) && 
@@ -931,11 +1035,12 @@ gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 }
 	
 gboolean 
-gedit_document_save (GeditDocument* doc, GError **error)
+gedit_document_save (GeditDocument *doc, GError **error)
 {	
 	gboolean auto_save;
 	gboolean create_backup_copy;
-
+	const GeditEncoding *encoding;
+	
 	gboolean ret;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
@@ -945,6 +1050,9 @@ gedit_document_save (GeditDocument* doc, GError **error)
 	g_return_val_if_fail (!doc->priv->readonly, FALSE);
 	g_return_val_if_fail (doc->priv->uri != NULL, FALSE);
 	
+	encoding = gedit_document_get_encoding (doc);
+	g_return_val_if_fail (encoding != NULL, FALSE);
+	
 	auto_save = gedit_prefs_manager_get_auto_save ();
 	create_backup_copy = gedit_prefs_manager_get_create_backup_copy ();
 		
@@ -959,6 +1067,7 @@ gedit_document_save (GeditDocument* doc, GError **error)
 	
 	ret = gedit_document_save_as_real (doc, 
 					   doc->priv->uri, 
+					   encoding,
 					   create_backup_copy, 
 					   error);
 
@@ -980,7 +1089,8 @@ gedit_document_save (GeditDocument* doc, GError **error)
 }
 
 gboolean	
-gedit_document_save_as (GeditDocument* doc, const gchar *uri, GError **error)
+gedit_document_save_as (GeditDocument* doc, const gchar *uri, 
+			const GeditEncoding *encoding, GError **error)
 {	
 	gboolean auto_save;
 	
@@ -992,6 +1102,9 @@ gedit_document_save_as (GeditDocument* doc, const gchar *uri, GError **error)
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
 
+	if (encoding == NULL)
+		encoding = gedit_document_get_encoding (doc);
+
 	auto_save = gedit_prefs_manager_get_auto_save ();
 
 	if (auto_save) 
@@ -1004,12 +1117,13 @@ gedit_document_save_as (GeditDocument* doc, const gchar *uri, GError **error)
 		}
 	}
 
-	if (gedit_document_save_as_real (doc, uri, TRUE, error))
+	if (gedit_document_save_as_real (doc, uri, encoding, TRUE, error))
 	{
 		gedit_document_set_uri (doc, uri);
 		gedit_document_set_readonly (doc, FALSE);
 		doc->priv->last_save_was_manually = TRUE;
-
+		gedit_document_set_encoding (doc, encoding);
+			
 		ret = TRUE;
 	}		
 
@@ -1027,7 +1141,8 @@ gedit_document_save_as (GeditDocument* doc, const gchar *uri, GError **error)
 }
 
 gboolean	
-gedit_document_save_a_copy_as (GeditDocument* doc, const gchar *uri, GError **error)
+gedit_document_save_a_copy_as (GeditDocument *doc, const gchar *uri, 
+			       const GeditEncoding *encoding, GError **error)
 {
 	gboolean m;
 	gboolean ret;
@@ -1037,10 +1152,13 @@ gedit_document_save_a_copy_as (GeditDocument* doc, const gchar *uri, GError **er
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
+
+	if (encoding == NULL)
+		encoding = gedit_document_get_encoding (doc);
 	
 	m = gedit_document_get_modified (doc);
 
-	ret = gedit_document_save_as_real (doc, uri, FALSE, error);
+	ret = gedit_document_save_as_real (doc, uri, encoding, FALSE, error);
 	
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), m);	 
 
@@ -1143,7 +1261,7 @@ follow_symlinks (const gchar *filename, GError **error)
  */
 
 static gboolean	
-gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
+gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEncoding *encoding,
 			     gboolean create_backup_copy, GError **error)
 {
 	gchar *filename; /* Filename without URI scheme */
@@ -1158,7 +1276,6 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	gint chars_len;
 	gint fd;
 	gint retval;
-	GeditSaveEncodingSetting encoding_setting;
 	gboolean res;
 	gboolean add_cr;
 	
@@ -1167,6 +1284,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
+	g_return_val_if_fail (encoding != NULL, FALSE);
 
 	retval = FALSE;
 
@@ -1276,22 +1394,25 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	}
 
 	chars = gedit_document_get_chars (doc, 0, -1);
-
-	encoding_setting = gedit_prefs_manager_get_save_encoding ();
-
-	if (encoding_setting == GEDIT_SAVE_CURRENT_LOCALE_IF_POSSIBLE)
+	
+	if (encoding != gedit_encoding_get_utf8 ())
 	{
 		GError *conv_error = NULL;
 		gchar* converted_file_contents = NULL;
 
-		gedit_debug (DEBUG_DOCUMENT, "Using current locale's encoding");
-
-		converted_file_contents = g_locale_from_utf8 (chars, -1, NULL, NULL, &conv_error);
+		converted_file_contents = gedit_convert_from_utf8 (chars, 
+								   -1, 
+								   encoding,
+								   &conv_error);
 
 		if (conv_error != NULL)
 		{
 			/* Conversion error */
-			g_error_free (conv_error);
+			g_propagate_error (error, conv_error);
+
+			close (fd);
+			unlink (temp_filename);
+			goto out;
 		}
 		else
 		{
@@ -1299,37 +1420,6 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 			chars = converted_file_contents;
 		}
 	}
-	else
-	{
-		if ((doc->priv->encoding != NULL) &&
-		    ((encoding_setting == GEDIT_SAVE_ORIGINAL_FILE_ENCODING_IF_POSSIBLE) ||
-		     (encoding_setting == GEDIT_SAVE_ORIGINAL_FILE_ENCODING_IF_POSSIBLE_NCL)))
-		{
-			GError *conv_error = NULL;
-			gchar* converted_file_contents = NULL;
-
-			gedit_debug (DEBUG_DOCUMENT, "Using encoding %s", doc->priv->encoding);
-
-			/* Try to convert it from UTF-8 to original encoding */
-			converted_file_contents = g_convert (chars, -1, 
-							     doc->priv->encoding, "UTF-8", 
-							     NULL, NULL, &conv_error); 
-
-			if (conv_error != NULL)
-			{
-				/* Conversion error */
-				g_error_free (conv_error);
-			}
-			else
-			{
-				g_free (chars);
-				chars = converted_file_contents;
-			}
-		}
-		else
-			gedit_debug (DEBUG_DOCUMENT, "Using UTF-8 (Null)");
-
-	}	    
 
 	chars_len = strlen (chars);
 
@@ -1528,7 +1618,7 @@ gedit_document_revert (GeditDocument *doc,  GError **error)
 
 	gedit_document_delete_text (doc, 0, -1);
 
-	if (!gedit_document_load (doc, doc->priv->uri, error))
+	if (!gedit_document_load (doc, doc->priv->uri, gedit_document_get_encoding (doc), error))
 	{
 		GtkTextIter iter;
 		
@@ -1655,6 +1745,20 @@ gedit_document_can_redo (const GeditDocument *doc)
 	return gtk_source_buffer_can_redo (GTK_SOURCE_BUFFER (doc));
 }
 
+/**
+ *  A function to determine whether or not the "FindNext/Prev"
+ *  functions are executable
+ */
+gboolean
+gedit_document_can_find_again (const GeditDocument *doc)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
+
+	return (doc->priv->last_searched_text != NULL);
+}
+
 void 
 gedit_document_undo (GeditDocument *doc)
 {
@@ -1791,18 +1895,17 @@ gedit_document_set_last_replace_text (GeditDocument* doc, const gchar *text)
 }
 
 gboolean
-gedit_document_find (GeditDocument* doc, const gchar* str, 
-		gboolean from_cursor, gboolean case_sensitive, gboolean entire_word)
+gedit_document_find (GeditDocument* doc, const gchar* str, gint flags)
 {
 	GtkTextIter iter;
 	gboolean found = FALSE;
-	GtkTextSearchFlags search_flags;
+	GtkSourceSearchFlags search_flags;
 	gchar *converted_str;
+	gboolean is_first_find = FALSE;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
-	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (str != NULL, FALSE);
 
 	converted_str = gedit_utils_convert_search_text (str);
@@ -1811,33 +1914,45 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 	gedit_debug (DEBUG_DOCUMENT, "str: %s", str);
 	gedit_debug (DEBUG_DOCUMENT, "converted_str: %s", converted_str);
 
-	search_flags = GTK_TEXT_SEARCH_VISIBLE_ONLY | GTK_TEXT_SEARCH_TEXT_ONLY;
+	search_flags = GTK_SOURCE_SEARCH_VISIBLE_ONLY | GTK_SOURCE_SEARCH_TEXT_ONLY;
 	
-	if (!case_sensitive)
+	if (!GEDIT_SEARCH_IS_CASE_SENSITIVE (flags))
 	{
-		search_flags = search_flags | GTK_TEXT_SEARCH_CASE_INSENSITIVE;
+		search_flags = search_flags | GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
 	}
 
-	if (from_cursor)
+	if (GEDIT_SEARCH_IS_FROM_CURSOR (flags))
 	{
 		GtkTextIter sel_bound;
 		
-		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &iter,
+		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc), &iter,
                                     gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
 					                      "insert"));
 		
-		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &sel_bound,
+		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc), &sel_bound,
                                     gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
 					                      "selection_bound"));
 		
-		gtk_text_iter_order (&sel_bound, &iter);		
+		if (!GEDIT_SEARCH_IS_BACKWARDS (flags))
+			gtk_text_iter_order (&sel_bound, &iter);		
+		else
+			gtk_text_iter_order (&iter, &sel_bound);		
 	}
 	else		
-		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, 0);
-
-	found = FALSE;
+	{
+		if (GEDIT_SEARCH_IS_BACKWARDS (flags))
+		{
+			/* get an iterator at the end of the document */
+			gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc),
+							    &iter, -1);
+		}
+		else
+		{
+			/* get an iterator at the beginning of the document */
+			gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc),
+							    &iter, 0);
+		}
+	}
 	
 	if (*converted_str != '\0')
     	{
@@ -1845,18 +1960,30 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 
 		while (!found)
 		{
-          		found = gedit_text_iter_forward_search (&iter, converted_str, search_flags,
+			if (GEDIT_SEARCH_IS_BACKWARDS (flags))
+			{
+				/*gtk_text_iter_backward_char (&iter);*/
+				
+	          		found = gtk_source_iter_backward_search (&iter,
+							converted_str, search_flags,
                         	                	&match_start, &match_end,
                                 	               	NULL);	
+			}
+			else
+			{
+	          		found = gtk_source_iter_forward_search (&iter,
+							converted_str, search_flags,
+                        	                	&match_start, &match_end,
+                                	               	NULL);
+			}
 
-			if (found && entire_word)
+			if (found && GEDIT_SEARCH_IS_ENTIRE_WORD (flags))
 			{
 				found = gtk_text_iter_starts_word (&match_start) && 
 					gtk_text_iter_ends_word (&match_end);
 
 				if (!found) 
 					iter = match_end;
-
 			}
 			else
 				break;
@@ -1871,21 +1998,34 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 					"selection_bound", &match_end);
 		}
 
-		if (doc->priv->last_searched_text)
+		if (gedit_document_can_find_again (doc))
+		{
 			g_free (doc->priv->last_searched_text);
+		}
+		else
+		{
+			is_first_find = TRUE;
+		}
 
 		doc->priv->last_searched_text = g_strdup (str);
-		doc->priv->last_search_was_case_sensitive = case_sensitive;
-		doc->priv->last_search_was_entire_word = entire_word;
+		doc->priv->last_search_was_case_sensitive = GEDIT_SEARCH_IS_CASE_SENSITIVE (flags);
+		doc->priv->last_search_was_entire_word = GEDIT_SEARCH_IS_ENTIRE_WORD (flags);
+
+		if (is_first_find)
+		{
+			g_signal_emit (G_OBJECT (doc), document_signals[CAN_FIND_AGAIN], 0);
+		}
 	}
 
 	g_free (converted_str);
 
+	g_signal_emit (G_OBJECT (doc), document_signals[CAN_FIND_AGAIN], 0);
+
 	return found;
 }
 
-gboolean
-gedit_document_find_again (GeditDocument* doc, gboolean from_cursor)
+static gboolean
+gedit_document_find_again (GeditDocument* doc, gint flags)
 {
 	gchar* last_searched_text;
 	gboolean found;
@@ -1900,17 +2040,41 @@ gedit_document_find_again (GeditDocument* doc, gboolean from_cursor)
 	if (last_searched_text == NULL)
 		return FALSE;
 	
-	found = gedit_document_find (doc, 
-				     last_searched_text, 
-				     from_cursor, 
-				     doc->priv->last_search_was_case_sensitive, 
-				     doc->priv->last_search_was_entire_word);
+	/* pack bitflag array with previous data */
+	GEDIT_SEARCH_SET_CASE_SENSITIVE (flags, doc->priv->last_search_was_case_sensitive);
+	GEDIT_SEARCH_SET_ENTIRE_WORD (flags, doc->priv->last_search_was_entire_word);
+
+	/* run the search */
+	found = gedit_document_find (doc, last_searched_text, flags);
 
 	g_free (last_searched_text);
 
 	return found;
 }
 
+gboolean 
+gedit_document_find_next (GeditDocument* doc, gint flags)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	GEDIT_SEARCH_SET_BACKWARDS (flags, FALSE);
+
+	/* run the search */
+	return gedit_document_find_again (doc, flags);
+}
+
+gboolean 
+gedit_document_find_prev (GeditDocument* doc, gint flags)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	/* pack bitflag array with the backwards searching operative */
+	GEDIT_SEARCH_SET_BACKWARDS (flags, TRUE);
+
+	/* run the search */
+	return gedit_document_find_again (doc, flags);
+}
+
 gboolean 
 gedit_document_get_selection (GeditDocument *doc, gint *start, gint *end)
 {
@@ -2000,9 +2164,9 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 
 gboolean
 gedit_document_replace_all (GeditDocument *doc,
-	      	const gchar *find, const gchar *replace, gboolean case_sensitive, gboolean entire_word)
+			    const gchar *find, const gchar *replace,
+			    gint flags)
 {
-	gboolean from_cursor = FALSE;
 	gboolean cont = 0;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
@@ -2013,11 +2177,15 @@ gedit_document_replace_all (GeditDocument *doc,
 
 	gedit_document_begin_user_action (doc);
 
-	while (gedit_document_find (doc, find, from_cursor, case_sensitive, entire_word)) 
+	GEDIT_SEARCH_SET_BACKWARDS (flags, FALSE);
+	GEDIT_SEARCH_SET_FROM_CURSOR (flags, FALSE);
+
+	while (gedit_document_find (doc, find, flags)) 
 	{
 		gedit_document_replace_selected_text (doc, replace);
 		
-		from_cursor = TRUE;
+		GEDIT_SEARCH_SET_FROM_CURSOR (flags, TRUE);
+		
 		++cont;
 	}
 
@@ -2130,7 +2298,7 @@ gedit_document_get_language (GeditDocument *doc)
 	return gtk_source_buffer_get_language (GTK_SOURCE_BUFFER (doc));
 }
 
-const GeditEncoding  *
+const GeditEncoding *
 gedit_document_get_encoding (GeditDocument *doc)
 {
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
@@ -2138,3 +2306,22 @@ gedit_document_get_encoding (GeditDocument *doc)
 	return gedit_encoding_get_from_charset (doc->priv->encoding);
 }
 
+static void
+gedit_document_set_encoding (GeditDocument *doc, const GeditEncoding *encoding)
+{
+	g_return_if_fail (encoding != NULL);
+
+	if (doc->priv->encoding != NULL)
+		g_free (doc->priv->encoding);
+
+	doc->priv->encoding = g_strdup (gedit_encoding_get_charset (encoding));
+
+	gedit_metadata_manager_set (doc->priv->uri,
+				    "encoding",
+				    doc->priv->encoding);
+
+	/* FIXME: do we need a encoding changed signal ? - Paolo */
+	g_signal_emit (G_OBJECT (doc), document_signals[NAME_CHANGED], 0);
+
+}
+
