@@ -3,7 +3,7 @@
 #ifndef __GEDIT_RECENT_H__
 #define __GEDIT_RECENT_H__
 
-#include "recent-files/egg-recent-model.h"
+#include <gedit/recent-files/egg-recent-model.h>
 
 
 EggRecentModel * gedit_recent_get_model (void);
