@@ -49,6 +49,7 @@
 #include "gedit-plugins-engine.h"
 #include "gedit-application-server.h"
 #include "gedit-convert.h"
+#include "gedit-metadata-manager.h"
 
 #ifndef GNOME_ICONDIR
 #define GNOME_ICONDIR "" 
@@ -79,6 +80,9 @@ static const struct poptOption options [] =
 	{ "debug-utils", '\0', POPT_ARG_NONE, &debug_utils, 0,
 	  N_("Show utility debugging messages."), NULL },
 
+	{ "debug-metadata", '\0', POPT_ARG_NONE, &debug_utils, 0,
+	  N_("Show metadata debugging messages."), NULL },
+
 	{ "debug-mdi", '\0', POPT_ARG_NONE, &debug_mdi, 0,
 	  N_("Show mdi debugging messages."), NULL },
 
@@ -281,18 +285,34 @@ gedit_handle_automation_cmdline (GnomeProgram *program)
 
 	if (stdin_data != NULL && strlen (stdin_data) > 0)
 	{
+		GError *conv_error = NULL;
+		const GeditEncoding *encoding;
 		gchar *converted_text;
 
-		if (g_utf8_validate (stdin_data, -1, NULL))
+		if (gedit_encoding_get_current () != gedit_encoding_get_utf8 ())
 		{
-			converted_text = stdin_data;
-			stdin_data = NULL;
+			if (g_utf8_validate (stdin_data, -1, NULL))
+			{
+				converted_text = stdin_data;
+				stdin_data = NULL;
+
+				encoding = gedit_encoding_get_utf8 ();
+			}
+			else
+			{
+				converted_text = NULL;
+			}
 		}
 		else
+		{	
+			encoding = gedit_encoding_get_current ();
+				
 			converted_text = gedit_convert_to_utf8 (stdin_data,
-							-1,
-							NULL);
-
+								-1,
+								&encoding,
+								&conv_error);
+		}	
+		
 		if (converted_text != NULL)
 		{
 			window = GNOME_Gedit_Application_getActiveWindow (server, &env);
@@ -367,10 +387,13 @@ main (int argc, char **argv)
 	/* Load user preferences */
 	gedit_prefs_manager_app_init ();
 	gedit_recent_init ();
+	gedit_metadata_manager_init ();
 
 	/* Init plugins engine */
 	gedit_plugins_engine_init ();
 
+	gnome_authentication_manager_init ();
+
 	/* Initialize session management */
 	gedit_session_init (argv[0]);
 
